@@ -33,7 +33,8 @@ mkdir ${TMPDIR} && \
 tar -C ${TMPDIR} -xzf $1 && \
 
 cat ${TMPDIR}/${SWAMIDIR}/swami.spec \
-    | sed 's|^\./configure.*$|& --with-audiofile|' \
+    | sed 's|^\./configure.*$|& --disable-alsa-support --with-audiofile'\
+' --disable-awe-caching|' \
     >${TMPDIR}/swami-binary-rpm.spec && \
 
 mv ${TMPDIR}/${SWAMIDIR}/swami.spec ${TMPDIR}/swami-binary-rpm.spec \
