@@ -76,18 +76,32 @@ enum
 #define SWAMIGUI_ROOT_DEFAULT_UPPER_KEYS  "q,2,w,3,e,r,5,t,6,y,7,u,i,9,o,0,p,bracketleft,equal,bracketright"
 
 /* external private global prototypes */
+void  swamigui_util_destroy_unique_dialog(void);
+
 void _swamigui_ifaces_init(void);  /* ifaces/ifaces.c */
 void _swamigui_stock_icons_init(void);  /* icons.c */
 void _swamigui_control_init(void);  /* SwamiguiControl.c */
 void _swamigui_control_widgets_init(void);  /* SwamiguiControl_widgets.c */
 void _swamigui_item_menu_init(void);
+void _patch_funcs_init(void); /* General instrument patch functions */
+void _swamigui_knob_init(void);
+void _swamigui_pref_init(void); /* Preferences functions */
+void _swamigui_sample_editor_init(void);
+void _swamigui_splits_init(void);
 void _swamigui_item_menu_actions_init(void);	/* SwamiguiItemMenu_actions.c */
+void _swamigui_tree_store_init(void);
 void _swamigui_panel_selector_init(void);
 void _swamigui_prop_init(void);
 void swami_deinit(void); /* free libswami  */
 void swamigui_util_deinit(void);
 void _swamigui_control_deinit(void); /* free GUI control system */
 void _swamigui_item_menu_deinit(void);
+void _patch_funcs_deinit(void); /* General instrument patch functions */
+void _swamigui_pref_deinit(void);
+void _swamigui_sample_editor_deinit(void);
+void _swamigui_splits_deinit(void);
+void _swamigui_knob_deinit(void);
+void _swamigui_tree_store_deinit(void);
 void _swamigui_panel_selector_deinit(void);
 void _swamigui_prop_deinit(void);
 void  swami_plugin_unload_all(void);
@@ -227,6 +241,8 @@ swamigui_init(int *argc, char **argv[])
     _swamigui_control_init();
     _swamigui_control_widgets_init();
     _swamigui_item_menu_init();
+    /* General instrument patch functions */
+    _patch_funcs_init();
 
 
     /* initialize Swamigui types */
@@ -240,6 +256,8 @@ swamigui_init(int *argc, char **argv[])
     swamigui_control_rank_get_type();
     swamigui_item_menu_flags_get_type();
     swamigui_item_menu_get_type();
+    /* knob widget */
+    _swamigui_knob_init();
     swamigui_knob_get_type();
     swamigui_menu_get_type();
     swamigui_mod_edit_get_type();
@@ -250,23 +268,29 @@ swamigui_init(int *argc, char **argv[])
     swamigui_paste_get_type();
     swamigui_paste_status_get_type();
     swamigui_piano_get_type();
+    /* preferences functions */
+    _swamigui_pref_init();
     swamigui_pref_get_type();
     swamigui_prop_get_type();
     swamigui_quit_confirm_get_type();
     swamigui_root_get_type();
     swamigui_sample_canvas_get_type();
+    /* Sample editor functions */
+    _swamigui_sample_editor_init();
     swamigui_sample_editor_get_type();
     swamigui_sample_editor_marker_flags_get_type();
     swamigui_sample_editor_marker_id_get_type();
     swamigui_sample_editor_status_get_type();
     swamigui_spectrum_canvas_get_type();
     swamigui_spin_scale_get_type();
+    _swamigui_splits_init();
     swamigui_splits_get_type();
     swamigui_splits_mode_get_type();
     swamigui_splits_status_get_type();
     swamigui_statusbar_get_type();
     swamigui_statusbar_pos_get_type();
     swamigui_tree_get_type();
+    _swamigui_tree_store_init();
     swamigui_tree_store_get_type();
     swamigui_tree_store_patch_get_type();
     swamigui_tree_store_config_get_type();
@@ -338,6 +362,24 @@ swamigui_deinit(void)
     /* Free pannel list */
     _swamigui_panel_selector_deinit();
 
+    /* free SwamiguiTreeStore functions */
+    _swamigui_tree_store_deinit();
+
+    /* Free SwamiguiSplits functions */
+    _swamigui_splits_deinit();
+
+    /* Free Sample editor functions */
+    _swamigui_sample_editor_deinit();
+
+    /* free preferences functions */
+    _swamigui_pref_deinit();
+
+    /* free knob widget functions */
+    _swamigui_knob_deinit();
+
+    /* free General instrument patch functions */
+    _patch_funcs_deinit();
+
     _swamigui_item_menu_deinit();
 
     /* Free GUI control system */
@@ -1086,6 +1128,9 @@ swamigui_root_quit(SwamiguiRoot *root)
     int quit_confirm;
     char *s;
 
+    /* destroy any dialog actally open */
+    swamigui_util_destroy_unique_dialog();
+
     /* Look among "patch items" if one of them has been changed */
     list = swami_root_get_patch_items(SWAMI_ROOT(root));   /* ++ ref list */
     ipatch_list_init_iter(list, &iter);
@@ -1434,6 +1479,71 @@ swamigui_root_load_prefs(SwamiguiRoot *root)
     return (TRUE);
 }
 
+/**
+ * swamigui_root_patch_load:
+ *
+ * Load an instrument patch file and displays a log error message if it fails.
+ * The log message is displayed on the console and optionally on a Gtk message
+ * dialog.
+ *
+ * In the case of a file loaded with error, the message displayed
+ * is critical.
+ * Because the function can ignore a file already loaded, for this case
+ * the error returned is not critical and the message displayed is an
+ * information only.
+ *
+ * @root: Swami root object to load into.
+ * @filename, Name and path of file to load.
+ * @item: Location to store pointer to object that has been loaded into Swami
+ *   root object (or NULL). Remember to unref the object when done with it
+ *   (not necessary of course if NULL was passed).
+ * @parent: window parent of the Gtk message dialog created to display the log
+ *  message. If NULL no message dialog is created.
+ * @return: TRUE on success, FALSE otherwise
+ */
+gboolean swamigui_root_patch_load(SwamiRoot *root, const char *filename,
+                                  IpatchItem **item, GtkWindow *parent)
+{
+    GError *err = NULL;
+
+    g_return_val_if_fail((parent == NULL ) || GTK_IS_WINDOW(parent), FALSE);
+
+    if (!swami_root_patch_load (root, filename, item, &err))
+    {
+        /* prepare a CRITICAL log message */
+        GLogLevelFlags log_level = G_LOG_LEVEL_CRITICAL;
+        static const char *log_msg = "Failed to load file '%s': %s";
+        GtkMessageType gtk_type = GTK_MESSAGE_ERROR;
+
+        if(err && err->code == SWAMI_ERROR_ALREADY_LOADED)
+        {
+            /* prepare an INFO log message */
+            log_level = G_LOG_LEVEL_INFO;
+            log_msg = "Ignore file '%s': %s";
+            gtk_type = GTK_MESSAGE_INFO;
+        }
+
+        /* displaying log message on the console */
+        g_log(G_LOG_DOMAIN, log_level, log_msg, filename,
+              ipatch_gerror_message(err));
+
+        /* optionally displaying log message on a GUI dialog */
+        if(parent)
+        {
+            GtkWidget *msgdialog;
+            msgdialog = gtk_message_dialog_new(GTK_WINDOW(parent), 0, gtk_type,
+                                               GTK_BUTTONS_OK, log_msg, filename,
+                                               ipatch_gerror_message(err));
+            gtk_dialog_run(GTK_DIALOG(msgdialog));
+            gtk_widget_destroy(msgdialog);
+        }
+
+        g_clear_error(&err);
+        return FALSE;
+    }
+    return TRUE;
+}
+
 /**
  * swamigui_get_root:
  * @gobject: An object registered to a #SwamiguiRoot object
@@ -1976,7 +2086,7 @@ sli_inst_prop_handler(GtkWidget *widg, GObject *obj)
         /* ++ ref new store */
         model = GTK_TREE_MODEL(gtk_tree_store_new(1, G_TYPE_STRING));
         cat_map_to_tree_store(GTK_TREE_STORE(model),
-                              ipatch_sli_inst_cat_map, NULL);
+                              ipatch_sli_inst_get_cat_map(), NULL);
         gtk_combo_box_set_model(GTK_COMBO_BOX(combo), model);
         g_object_unref(model);  /* -- unref store */
         /* set cell renderer */
@@ -2018,6 +2128,7 @@ cat_map_to_tree_store(GtkTreeStore *store, const IpatchSLIInstCatMapEntry *catma
     GtkTreeIter child;
     guint i;
     gchar *label, *prefix;
+    const gchar **inst_cat_strings = ipatch_sli_inst_get_cat_strings();
 
     g_return_if_fail(store != NULL && catmap != NULL);
 
@@ -2027,12 +2138,12 @@ cat_map_to_tree_store(GtkTreeStore *store, const IpatchSLIInstCatMapEntry *catma
         {
             gtk_tree_model_get(GTK_TREE_MODEL(store), parent, 0, &prefix, -1);
             label = g_strjoin(" | ", prefix,
-                              ipatch_sli_inst_cat_strings[catmap[i].name_idx], NULL);
+                              inst_cat_strings[catmap[i].name_idx], NULL);
             g_free(prefix);
         }
         else
         {
-            label = g_strdup(ipatch_sli_inst_cat_strings[catmap[i].name_idx]);
+            label = g_strdup (inst_cat_strings[catmap[i].name_idx]);
         }
 
         gtk_tree_store_append(store, &child, parent);
@@ -2050,7 +2161,7 @@ cat_map_to_tree_store(GtkTreeStore *store, const IpatchSLIInstCatMapEntry *catma
     {
         gtk_tree_store_append(store, &child, parent);
         gtk_tree_store_set(store, &child, 0,
-                           ipatch_sli_inst_cat_strings[catmap[i].name_idx], -1);
+                           inst_cat_strings[catmap[i].name_idx], -1);
     }
 }
 
@@ -2058,7 +2169,7 @@ cat_map_to_tree_store(GtkTreeStore *store, const IpatchSLIInstCatMapEntry *catma
 static void
 category_changed_cb(GtkComboBox *combo, gpointer user_data)
 {
-    const IpatchSLIInstCatMapEntry *catmap = ipatch_sli_inst_cat_map;
+    const IpatchSLIInstCatMapEntry *catmap = ipatch_sli_inst_get_cat_map();
     IpatchSLIInst *inst;
     gboolean ret;
     GtkTreeModel *model;
